@@ -23,10 +23,11 @@
 #ifndef __GST_I18N_APP_H__
 #define __GST_I18N_APP_H__
 
-#include "gettext.h" /* included with gettext distribution and copied */
-
 #ifdef ENABLE_NLS
 
+#include <locale.h>  /* some people need it and some people don't */
+#include "gettext.h" /* included with gettext distribution and copied */
+
 /* we want to use shorthand _() for translating and N_() for marking */
 #define _(String) gettext (String)
 #define N_(String) gettext_noop (String)
@@ -36,6 +37,7 @@
 
 #define _(String) String
 #define N_(String) String
+#define ngettext(Singular,Plural,Count) ((Count>1)?Plural:Singular)
 
 #endif
 
