@@ -84,6 +84,7 @@
  */
 
 #include "gst_private.h"
+#include "glib-compat-private.h"
 
 #include "gstobject.h"
 #include "gstmarshal.h"
@@ -104,7 +105,7 @@ enum
 {
   PARENT_SET,
   PARENT_UNSET,
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   OBJECT_SAVED,
 #endif
   DEEP_NOTIFY,
@@ -133,10 +134,8 @@ typedef struct _GstSignalObject GstSignalObject;
 typedef struct _GstSignalObjectClass GstSignalObjectClass;
 
 static GType gst_signal_object_get_type (void);
-static void gst_signal_object_class_init (GstSignalObjectClass * klass);
-static void gst_signal_object_init (GstSignalObject * object);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 static guint gst_signal_object_signals[SO_LAST_SIGNAL] = { 0 };
 #endif
 
@@ -152,9 +151,22 @@ static void gst_object_finalize (GObject * object);
 
 static gboolean gst_object_set_name_default (GstObject * object);
 
-#ifndef GST_DISABLE_LOADSAVE
+#ifdef GST_DISABLE_DEPRECATED
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
+#undef GstXmlNodePtr
+#define GstXmlNodePtr xmlNodePtr
+#include <libxml/parser.h>
+GstXmlNodePtr gst_object_save_thyself (GstObject * object,
+    GstXmlNodePtr parent);
+void gst_object_restore_thyself (GstObject * object, GstXmlNodePtr parent);
+void gst_class_signal_emit_by_name (GstObject * object, const gchar * name,
+    GstXmlNodePtr self);
+#endif
+#endif
+
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 static void gst_object_real_restore_thyself (GstObject * object,
-    xmlNodePtr self);
+    GstXmlNodePtr self);
 #endif
 
 static GObjectClass *parent_class = NULL;
@@ -173,8 +185,8 @@ gst_object_class_init (GstObjectClass * klass)
   _gst_object_trace = gst_alloc_trace_register (g_type_name (GST_TYPE_OBJECT));
 #endif
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_object_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_object_get_property);
+  gobject_class->set_property = gst_object_set_property;
+  gobject_class->get_property = gst_object_get_property;
 
   g_object_class_install_property (gobject_class, ARG_NAME,
       g_param_spec_string ("name", "Name", "The name of the object",
@@ -205,7 +217,7 @@ gst_object_class_init (GstObjectClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstObjectClass, parent_unset), NULL,
       NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_OBJECT);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   /**
    * GstObject::object-saved:
    * @gstobject: a #GstObject
@@ -222,7 +234,9 @@ gst_object_class_init (GstObjectClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstObjectClass, object_saved), NULL,
       NULL, g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-  klass->restore_thyself = gst_object_real_restore_thyself;
+  klass->restore_thyself =
+      ((void (*)(GstObject * object,
+              gpointer self)) *gst_object_real_restore_thyself);
 #endif
 
   /**
@@ -243,10 +257,11 @@ gst_object_class_init (GstObjectClass * klass)
       G_TYPE_PARAM);
 
   klass->path_string_separator = "/";
-  klass->lock = g_new0 (GStaticRecMutex, 1);
+  /* FIXME 0.11: Store this directly in the class struct */
+  klass->lock = g_slice_new (GStaticRecMutex);
   g_static_rec_mutex_init (klass->lock);
 
-  klass->signal_object = g_object_new (gst_signal_object_get_type (), NULL);
+  klass->signal_object = g_object_newv (gst_signal_object_get_type (), 0, NULL);
 
   /* see the comments at gst_object_dispatch_properties_changed */
   gobject_class->dispatch_properties_changed
@@ -262,7 +277,7 @@ gst_object_init (GstObject * object)
   object->lock = g_mutex_new ();
   object->parent = NULL;
   object->name = NULL;
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p new", object);
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p new", object);
 
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_new (_gst_object_trace, object);
@@ -284,7 +299,7 @@ gst_object_init (GstObject * object)
  * constructs like :
  *  result = gst_object_ref (object->parent);
  *
- * Returns: A pointer to @object
+ * Returns: (transfer full): A pointer to @object
  */
 gpointer
 gst_object_ref (gpointer object)
@@ -292,8 +307,7 @@ gst_object_ref (gpointer object)
   g_return_val_if_fail (object != NULL, NULL);
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p ref %d->%d",
-      object,
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p ref %d->%d", object,
       ((GObject *) object)->ref_count, ((GObject *) object)->ref_count + 1);
 #endif
   g_object_ref (object);
@@ -319,13 +333,46 @@ gst_object_unref (gpointer object)
   g_return_if_fail (((GObject *) object)->ref_count > 0);
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p unref %d->%d",
-      object,
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p unref %d->%d", object,
       ((GObject *) object)->ref_count, ((GObject *) object)->ref_count - 1);
 #endif
   g_object_unref (object);
 }
 
+/**
+ * gst_object_ref_sink:
+ * @object: a #GstObject to sink
+ *
+ * Increase the reference count of @object, and possibly remove the floating
+ * reference, if @object has a floating reference.
+ *
+ * In other words, if the object is floating, then this call "assumes ownership"
+ * of the floating reference, converting it to a normal reference by clearing
+ * the floating flag while leaving the reference count unchanged. If the object
+ * is not floating, then this call adds a new normal reference increasing the
+ * reference count by one.
+ *
+ * MT safe. This function grabs and releases @object lock.
+ *
+ * Since: 0.10.24
+ */
+void
+gst_object_ref_sink (gpointer object)
+{
+  g_return_if_fail (GST_IS_OBJECT (object));
+
+  GST_OBJECT_LOCK (object);
+  if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object,
+        "unsetting floating flag");
+    GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
+    GST_OBJECT_UNLOCK (object);
+  } else {
+    GST_OBJECT_UNLOCK (object);
+    gst_object_ref (object);
+  }
+}
+
 /**
  * gst_object_sink:
  * @object: a #GstObject to sink
@@ -348,11 +395,11 @@ gst_object_sink (gpointer object)
 {
   g_return_if_fail (GST_IS_OBJECT (object));
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
 
   GST_OBJECT_LOCK (object);
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "clear floating flag");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "clear floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
     gst_object_unref (object);
@@ -363,8 +410,9 @@ gst_object_sink (gpointer object)
 
 /**
  * gst_object_replace:
- * @oldobj: pointer to a place of a #GstObject to replace
- * @newobj: a new #GstObject
+ * @oldobj: (inout) (transfer full): pointer to a place of a #GstObject to
+ *     replace
+ * @newobj: (transfer none): a new #GstObject
  *
  * Unrefs the #GstObject pointed to by @oldobj, refs @newobj and
  * puts @newobj in *@oldobj. Be carefull when calling this
@@ -374,30 +422,33 @@ gst_object_sink (gpointer object)
  *
  * Make sure not to LOCK @oldobj because it might be unreffed
  * which could cause a deadlock when it is disposed.
+ *
+ * Since 0.10.36, this function operates atomically.
  */
 void
 gst_object_replace (GstObject ** oldobj, GstObject * newobj)
 {
+  GstObject *oldptr;
+
   g_return_if_fail (oldobj != NULL);
   g_return_if_fail (*oldobj == NULL || GST_IS_OBJECT (*oldobj));
   g_return_if_fail (newobj == NULL || GST_IS_OBJECT (newobj));
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %s (%d) with %s (%d)",
-      *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)",
+  GST_CAT_TRACE (GST_CAT_REFCOUNTING, "replace %p %s (%d) with %p %s (%d)",
+      *oldobj, *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)",
       *oldobj ? G_OBJECT (*oldobj)->ref_count : 0,
-      newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)",
+      newobj, newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)",
       newobj ? G_OBJECT (newobj)->ref_count : 0);
 #endif
 
-  if (G_LIKELY (*oldobj != newobj)) {
-    if (newobj)
-      gst_object_ref (newobj);
-    if (*oldobj)
-      gst_object_unref (*oldobj);
-
-    *oldobj = newobj;
-  }
+  if (newobj)
+    g_object_ref (newobj);
+  do {
+    oldptr = *oldobj;
+  } while (!G_ATOMIC_POINTER_COMPARE_AND_EXCHANGE (oldobj, oldptr, newobj));
+  if (oldptr)
+    g_object_unref (oldptr);
 }
 
 /* dispose is called when the object has to release all links
@@ -407,7 +458,7 @@ gst_object_dispose (GObject * object)
 {
   GstObject *parent;
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
 
   GST_OBJECT_LOCK (object);
   if ((parent = GST_OBJECT_PARENT (object)))
@@ -428,7 +479,7 @@ have_parent:
         GST_OBJECT_NAME (object), GST_OBJECT_NAME (parent));
     GST_OBJECT_UNLOCK (object);
     /* ref the object again to revive it in this error case */
-    object = gst_object_ref (object);
+    gst_object_ref (object);
     return;
   }
 }
@@ -437,9 +488,9 @@ have_parent:
 static void
 gst_object_finalize (GObject * object)
 {
-  GstObject *gstobject = GST_OBJECT (object);
+  GstObject *gstobject = GST_OBJECT_CAST (object);
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
 
   g_signal_handlers_destroy (object);
 
@@ -453,7 +504,7 @@ gst_object_finalize (GObject * object)
   parent_class->finalize (object);
 }
 
-/* Changing a GObject property of a GstObject will result in "deep_notify"
+/* Changing a GObject property of a GstObject will result in "deep-notify"
  * signals being emitted by the object itself, as well as in each parent
  * object. This is so that an application can connect a listener to the
  * top-level bin to catch property-change notifications for all contained
@@ -470,18 +521,22 @@ gst_object_dispatch_properties_changed (GObject * object,
 {
   GstObject *gst_object, *parent, *old_parent;
   guint i;
-  gchar *name, *debug_name;
-
-  /* we fail when this is not a GstObject */
-  g_return_if_fail (GST_IS_OBJECT (object));
+#ifndef GST_DISABLE_GST_DEBUG
+  gchar *name = NULL;
+  const gchar *debug_name;
+#endif
 
   /* do the standard dispatching */
-  G_OBJECT_CLASS (parent_class)->dispatch_properties_changed (object, n_pspecs,
-      pspecs);
+  parent_class->dispatch_properties_changed (object, n_pspecs, pspecs);
 
   gst_object = GST_OBJECT_CAST (object);
-  name = gst_object_get_name (gst_object);
-  debug_name = GST_STR_NULL (name);
+#ifndef GST_DISABLE_GST_DEBUG
+  if (G_UNLIKELY (__gst_debug_min >= GST_LEVEL_LOG)) {
+    name = gst_object_get_name (gst_object);
+    debug_name = GST_STR_NULL (name);
+  } else
+    debug_name = "";
+#endif
 
   /* now let the parent dispatch those, too */
   parent = gst_object_get_parent (gst_object);
@@ -491,15 +546,16 @@ gst_object_dispatch_properties_changed (GObject * object,
           "deep notification from %s (%s)", debug_name, pspecs[i]->name);
 
       g_signal_emit (parent, gst_object_signals[DEEP_NOTIFY],
-          g_quark_from_string (pspecs[i]->name), GST_OBJECT_CAST (object),
-          pspecs[i]);
+          g_quark_from_string (pspecs[i]->name), gst_object, pspecs[i]);
     }
 
     old_parent = parent;
     parent = gst_object_get_parent (old_parent);
     gst_object_unref (old_parent);
   }
+#ifndef GST_DISABLE_GST_DEBUG
   g_free (name);
+#endif
 }
 
 /**
@@ -507,8 +563,9 @@ gst_object_dispatch_properties_changed (GObject * object,
  * @object: the #GObject that signalled the notify.
  * @orig: a #GstObject that initiated the notify.
  * @pspec: a #GParamSpec of the property.
- * @excluded_props: a set of user-specified properties to exclude or
- *  NULL to show all changes.
+ * @excluded_props: (array zero-terminated=1) (element-type gchar*)
+ *     (allow-none):a set of user-specified properties to exclude or
+ *     NULL to show all changes.
  *
  * A default deep_notify signal callback for an object. The user data
  * should contain a pointer to an array of strings that should be excluded
@@ -533,7 +590,7 @@ gst_object_default_deep_notify (GObject * object, GstObject * orig,
         return;
       excluded_props++;
     }
-    g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+    g_value_init (&value, pspec->value_type);
     g_object_get_property (G_OBJECT (orig), pspec->name, &value);
 
     /* FIXME: handle flags */
@@ -566,9 +623,9 @@ gst_object_set_name_default (GstObject * object)
 {
   const gchar *type_name;
   gint count;
-  gchar *name, *tmp;
-  gboolean result;
+  gchar *name;
   GQuark q;
+  guint i, l;
 
   /* to ensure guaranteed uniqueness across threads, only one thread
    * may ever assign a name */
@@ -584,18 +641,33 @@ gst_object_set_name_default (GstObject * object)
 
   G_UNLOCK (object_name_mutex);
 
-  /* GstFooSink -> foosinkN */
+  /* GstFooSink -> foosink<N> */
   type_name = g_quark_to_string (q);
   if (strncmp (type_name, "Gst", 3) == 0)
     type_name += 3;
-  tmp = g_strdup_printf ("%s%d", type_name, count);
-  name = g_ascii_strdown (tmp, -1);
-  g_free (tmp);
+  name = g_strdup_printf ("%s%d", type_name, count);
+  l = strlen (name);
+  for (i = 0; i < l; i++)
+    name[i] = g_ascii_tolower (name[i]);
 
-  result = gst_object_set_name (object, name);
-  g_free (name);
+  GST_OBJECT_LOCK (object);
+  if (G_UNLIKELY (object->parent != NULL))
+    goto had_parent;
 
-  return result;
+  g_free (object->name);
+  object->name = name;
+
+  GST_OBJECT_UNLOCK (object);
+
+  return TRUE;
+
+had_parent:
+  {
+    g_free (name);
+    GST_WARNING ("parented objects can't be renamed");
+    GST_OBJECT_UNLOCK (object);
+    return FALSE;
+  }
 }
 
 /**
@@ -636,6 +708,10 @@ gst_object_set_name (GstObject * object, const gchar * name)
     GST_OBJECT_UNLOCK (object);
     result = gst_object_set_name_default (object);
   }
+  /* FIXME-0.11: this misses a g_object_notify (object, "name"); unless called
+   * from gst_object_set_property.
+   * Ideally remove such custom setters (or make it static).
+   */
   return result;
 
   /* error */
@@ -656,7 +732,9 @@ had_parent:
  * For a nameless object, this returns NULL, which you can safely g_free()
  * as well.
  *
- * Returns: the name of @object. g_free() after usage.
+ * Free-function: g_free
+ *
+ * Returns: (transfer full): the name of @object. g_free() after usage.
  *
  * MT safe. This function grabs and releases @object's LOCK.
  */
@@ -684,7 +762,14 @@ gst_object_get_name (GstObject * object)
  * retains ownership of the name prefix it sent.
  *
  * MT safe.  This function grabs and releases @object's LOCK.
+ *
+ * Deprecated: deprecated because the name prefix has never actually been used
+ *     for anything.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+void gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix);
+#endif
 void
 gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
 {
@@ -695,6 +780,7 @@ gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
   object->name_prefix = g_strdup (name_prefix); /* NULL gives NULL */
   GST_OBJECT_UNLOCK (object);
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_object_get_name_prefix:
@@ -708,7 +794,14 @@ gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
  * Returns: the name prefix of @object. g_free() after usage.
  *
  * MT safe. This function grabs and releases @object's LOCK.
+ *
+ * Deprecated: deprecated because the name prefix has never actually been used
+ *     for anything.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+gchar *gst_object_get_name_prefix (GstObject * object);
+#endif
 gchar *
 gst_object_get_name_prefix (GstObject * object)
 {
@@ -722,6 +815,7 @@ gst_object_get_name_prefix (GstObject * object)
 
   return result;
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_object_set_parent:
@@ -758,7 +852,8 @@ gst_object_set_parent (GstObject * object, GstObject * parent)
    * in the floating case. */
   object->parent = parent;
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unsetting floating flag");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object,
+        "unsetting floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
   } else {
@@ -766,7 +861,7 @@ gst_object_set_parent (GstObject * object, GstObject * parent)
     gst_object_ref (object);
   }
 
-  g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_SET], 0, parent);
+  g_signal_emit (object, gst_object_signals[PARENT_SET], 0, parent);
 
   return TRUE;
 
@@ -787,8 +882,8 @@ had_parent:
  * Returns the parent of @object. This function increases the refcount
  * of the parent object so you should gst_object_unref() it after usage.
  *
- * Returns: parent of @object, this can be NULL if @object has no
- *   parent. unref after usage.
+ * Returns: (transfer full): parent of @object, this can be NULL if @object
+ *   has no parent. unref after usage.
  *
  * MT safe. Grabs and releases @object's LOCK.
  */
@@ -828,12 +923,11 @@ gst_object_unparent (GstObject * object)
   parent = object->parent;
 
   if (G_LIKELY (parent != NULL)) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
     object->parent = NULL;
     GST_OBJECT_UNLOCK (object);
 
-    g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_UNSET], 0,
-        parent);
+    g_signal_emit (object, gst_object_signals[PARENT_UNSET], 0, parent);
 
     gst_object_unref (object);
   } else {
@@ -847,7 +941,7 @@ gst_object_unparent (GstObject * object)
  * @ancestor: a #GstObject to check as ancestor
  *
  * Check if @object has an ancestor @ancestor somewhere up in
- * the hierarchy.
+ * the hierarchy. One can e.g. check if a #GstElement is inside a #GstPipeline.
  *
  * Returns: TRUE if @ancestor is an ancestor of @object.
  *
@@ -856,26 +950,30 @@ gst_object_unparent (GstObject * object)
 gboolean
 gst_object_has_ancestor (GstObject * object, GstObject * ancestor)
 {
-  GstObject *parent;
-  gboolean result = FALSE;
+  GstObject *parent, *tmp;
 
-  if (object == NULL)
+  if (!ancestor || !object)
     return FALSE;
 
-  if (object == ancestor)
-    return TRUE;
+  parent = gst_object_ref (object);
+  do {
+    if (parent == ancestor) {
+      gst_object_unref (parent);
+      return TRUE;
+    }
 
-  parent = gst_object_get_parent (object);
-  result = gst_object_has_ancestor (parent, ancestor);
-  if (parent)
+    tmp = gst_object_get_parent (parent);
     gst_object_unref (parent);
+    parent = tmp;
+  } while (parent);
 
-  return result;
+  return FALSE;
 }
 
 /**
  * gst_object_check_uniqueness:
- * @list: a list of #GstObject to check through
+ * @list: (transfer none) (element-type Gst.Object): a list of #GstObject to
+ *      check through
  * @name: the name to search for
  *
  * Checks to see if there is any object named @name in @list. This function
@@ -900,7 +998,7 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
     GstObject *child;
     gboolean eq;
 
-    child = GST_OBJECT (list->data);
+    child = GST_OBJECT_CAST (list->data);
 
     GST_OBJECT_LOCK (child);
     eq = strcmp (GST_OBJECT_NAME (child), name) == 0;
@@ -915,7 +1013,7 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
 }
 
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 /**
  * gst_object_save_thyself:
  * @object: a #GstObject to save
@@ -925,8 +1023,8 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
  *
  * Returns: the new xmlNodePtr with the saved object
  */
-xmlNodePtr
-gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
+GstXmlNodePtr
+gst_object_save_thyself (GstObject * object, GstXmlNodePtr parent)
 {
   GstObjectClass *oclass;
 
@@ -938,8 +1036,7 @@ gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
   if (oclass->save_thyself)
     oclass->save_thyself (object, parent);
 
-  g_signal_emit (G_OBJECT (object), gst_object_signals[OBJECT_SAVED], 0,
-      parent);
+  g_signal_emit (object, gst_object_signals[OBJECT_SAVED], 0, parent);
 
   return parent;
 }
@@ -952,7 +1049,7 @@ gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
  * Restores @object with the data from the parent XML node.
  */
 void
-gst_object_restore_thyself (GstObject * object, xmlNodePtr self)
+gst_object_restore_thyself (GstObject * object, GstXmlNodePtr self)
 {
   GstObjectClass *oclass;
 
@@ -966,7 +1063,7 @@ gst_object_restore_thyself (GstObject * object, xmlNodePtr self)
 }
 
 static void
-gst_object_real_restore_thyself (GstObject * object, xmlNodePtr self)
+gst_object_real_restore_thyself (GstObject * object, GstXmlNodePtr self)
 {
   g_return_if_fail (GST_IS_OBJECT (object));
   g_return_if_fail (self != NULL);
@@ -981,7 +1078,7 @@ gst_object_set_property (GObject * object, guint prop_id,
 {
   GstObject *gstobject;
 
-  gstobject = GST_OBJECT (object);
+  gstobject = GST_OBJECT_CAST (object);
 
   switch (prop_id) {
     case ARG_NAME:
@@ -999,7 +1096,7 @@ gst_object_get_property (GObject * object, guint prop_id,
 {
   GstObject *gstobject;
 
-  gstobject = GST_OBJECT (object);
+  gstobject = GST_OBJECT_CAST (object);
 
   switch (prop_id) {
     case ARG_NAME:
@@ -1018,7 +1115,9 @@ gst_object_get_property (GObject * object, guint prop_id,
  * Generates a string describing the path of @object in
  * the object hierarchy. Only useful (or used) for debugging.
  *
- * Returns: a string describing the path of @object. You must
+ * Free-function: g_free
+ *
+ * Returns: (transfer full): a string describing the path of @object. You must
  *          g_free() the string after usage.
  *
  * MT safe. Grabs and releases the #GstObject's LOCK for all objects
@@ -1033,7 +1132,7 @@ gst_object_get_path_string (GstObject * object)
   gchar *prevpath, *path;
   const gchar *typename;
   gchar *component;
-  gchar *separator;
+  const gchar *separator;
 
   /* ref object before adding to list */
   gst_object_ref (object);
@@ -1106,9 +1205,9 @@ struct _GstSignalObjectClass
   GObjectClass parent_class;
 
   /* signals */
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   void (*object_loaded) (GstSignalObject * object, GstObject * new,
-      xmlNodePtr self);
+      GstXmlNodePtr self);
 #endif
 };
 
@@ -1119,7 +1218,7 @@ gst_signal_object_class_init (GstSignalObjectClass * klass)
 {
   parent_class = g_type_class_peek_parent (klass);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   gst_signal_object_signals[SO_OBJECT_LOADED] =
       g_signal_new ("object-loaded", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstSignalObjectClass, object_loaded),
@@ -1154,7 +1253,7 @@ gst_class_signal_connect (GstObjectClass * klass,
       func_data);
 }
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 /**
  * gst_class_signal_emit_by_name:
  * @object: a #GstObject that emits the signal
@@ -1165,7 +1264,7 @@ gst_class_signal_connect (GstObjectClass * klass,
  */
 void
 gst_class_signal_emit_by_name (GstObject * object,
-    const gchar * name, xmlNodePtr self)
+    const gchar * name, GstXmlNodePtr self)
 {
   GstObjectClass *oclass;
 
