@@ -24,6 +24,13 @@
  * @see_also: #GstFileSrc
  *
  * Read data from a file in the local file system.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch filesrc location=song.ogg ! decodebin2 ! autoaudiosink
+ * ]| Play a song.ogg from local dir.
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -87,7 +94,7 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 /* Copy of glib's g_open due to win32 libc/cross-DLL brokenness: we can't
  * use the 'file descriptor' opened in glib (and returned from this function)
  * in this library, as they may have unrelated C runtimes. */
-int
+static int
 gst_open (const gchar * filename, int flags, int mode)
 {
 #ifdef G_OS_WIN32
@@ -291,7 +298,7 @@ gst_file_src_class_init (GstFileSrcClass * klass)
           DEFAULT_SEQUENTIAL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
           GST_PARAM_MUTABLE_PLAYING));
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_file_src_finalize);
+  gobject_class->finalize = gst_file_src_finalize;
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_file_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_file_src_stop);
@@ -360,10 +367,12 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
     src->filename = NULL;
     src->uri = NULL;
   } else {
-    /* we store the filename as received by the application. On Windoes this
+    /* we store the filename as received by the application. On Windows this
      * should be UTF8 */
     src->filename = g_strdup (location);
-    src->uri = gst_uri_construct ("file", src->filename);
+    src->uri = gst_filename_to_uri (location, NULL);
+    GST_INFO ("filename : %s", src->filename);
+    GST_INFO ("uri      : %s", src->uri);
   }
   g_object_notify (G_OBJECT (src), "location");
   gst_uri_handler_new_uri (GST_URI_HANDLER (src), src->uri);
@@ -373,7 +382,8 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
   /* ERROR */
 wrong_state:
   {
-    GST_DEBUG_OBJECT (src, "setting location in wrong state");
+    g_warning ("Changing the `location' property on filesrc when a file is "
+        "open is not supported.");
     GST_OBJECT_UNLOCK (src);
     return FALSE;
   }
@@ -485,52 +495,24 @@ struct _GstMmapBufferClass
   GstBufferClass buffer_class;
 };
 
-static void gst_mmap_buffer_init (GTypeInstance * instance, gpointer g_class);
-static void gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data);
 static void gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer);
-static GstBufferClass *mmap_buffer_parent_class = NULL;
 
-static GType
-gst_mmap_buffer_get_type (void)
-{
-  static GType _gst_mmap_buffer_type;
+GType gst_mmap_buffer_get_type (void);
 
-  if (G_UNLIKELY (_gst_mmap_buffer_type == 0)) {
-    static const GTypeInfo mmap_buffer_info = {
-      sizeof (GstMmapBufferClass),
-      NULL,
-      NULL,
-      gst_mmap_buffer_class_init,
-      NULL,
-      NULL,
-      sizeof (GstMmapBuffer),
-      0,
-      gst_mmap_buffer_init,
-      NULL
-    };
-
-    _gst_mmap_buffer_type = g_type_register_static (GST_TYPE_BUFFER,
-        "GstMmapBuffer", &mmap_buffer_info, 0);
-  }
-  return _gst_mmap_buffer_type;
-}
+G_DEFINE_TYPE (GstMmapBuffer, gst_mmap_buffer, GST_TYPE_BUFFER);
 
 static void
-gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data)
+gst_mmap_buffer_class_init (GstMmapBufferClass * g_class)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
-  mmap_buffer_parent_class = g_type_class_peek_parent (g_class);
-
   mini_object_class->finalize =
       (GstMiniObjectFinalizeFunction) gst_mmap_buffer_finalize;
 }
 
 static void
-gst_mmap_buffer_init (GTypeInstance * instance, gpointer g_class)
+gst_mmap_buffer_init (GstMmapBuffer * buf)
 {
-  GstBuffer *buf = (GstBuffer *) instance;
-
   GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_READONLY);
   /* before we re-enable this flag, we probably need to fix _copy()
    * _make_writable(), etc. in GstMiniObject/GstBuffer as well */
@@ -571,8 +553,8 @@ gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer)
   GST_LOG ("unmapped region %08lx+%08lx at %p",
       (gulong) offset, (gulong) size, data);
 
-  GST_MINI_OBJECT_CLASS (mmap_buffer_parent_class)->finalize (GST_MINI_OBJECT
-      (mmap_buffer));
+  GST_MINI_OBJECT_CLASS (gst_mmap_buffer_parent_class)->finalize
+      (GST_MINI_OBJECT (mmap_buffer));
 }
 
 static GstBuffer *
@@ -584,8 +566,9 @@ gst_file_src_map_region (GstFileSrc * src, off_t offset, gsize size,
 
   g_return_val_if_fail (offset >= 0, NULL);
 
-  GST_LOG_OBJECT (src, "mapping region %08llx+%08lx from file into memory",
-      offset, (gulong) size);
+  /* FIXME ? use goffset and friends if we require glib >= 2.20 */
+  GST_LOG_OBJECT (src, "mapping region %08" G_GINT64_MODIFIER "x+%08lx "
+      "from file into memory", (gint64) offset, (gulong) size);
 
   mmapregion = mmap (NULL, size, PROT_READ, MAP_SHARED, src->fd, offset);
 
@@ -780,11 +763,11 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
 
   /* if we need to touch the buffer (to bring it into memory), do so */
   if (src->touch) {
-    volatile guchar *p = GST_BUFFER_DATA (buf), c;
+    volatile guchar *p = GST_BUFFER_DATA (buf);
 
     /* read first byte of each page */
     for (i = 0; i < GST_BUFFER_SIZE (buf); i += src->pagesize)
-      c = p[i];
+      (void) p[i];
   }
 
   /* we're done, return the buffer */
@@ -817,6 +800,7 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
 {
   int ret;
   GstBuffer *buf;
+  guint to_read, bytes_read;
 
   if (G_UNLIKELY (src->read_position != offset)) {
     off_t res;
@@ -834,30 +818,42 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
     return GST_FLOW_ERROR;
   }
 
-  GST_LOG_OBJECT (src, "Reading %d bytes at offset 0x%" G_GINT64_MODIFIER "x",
-      length, offset);
-  ret = read (src->fd, GST_BUFFER_DATA (buf), length);
-  if (G_UNLIKELY (ret < 0))
-    goto could_not_read;
-
-  /* seekable regular files should have given us what we expected */
-  if (G_UNLIKELY ((guint) ret < length && src->seekable))
-    goto unexpected_eos;
-
-  /* other files should eos if they read 0 and more was requested */
-  if (G_UNLIKELY (ret == 0 && length > 0))
-    goto eos;
-
-  length = ret;
-
-  GST_BUFFER_SIZE (buf) = length;
+  /* No need to read anything if length is 0 */
+  GST_BUFFER_SIZE (buf) = 0;
   GST_BUFFER_OFFSET (buf) = offset;
-  GST_BUFFER_OFFSET_END (buf) = offset + length;
+  GST_BUFFER_OFFSET_END (buf) = offset;
+  bytes_read = 0;
+  to_read = length;
+  while (to_read > 0) {
+    GST_LOG_OBJECT (src, "Reading %d bytes at offset 0x%" G_GINT64_MODIFIER "x",
+        to_read, offset + bytes_read);
+    errno = 0;
+    ret = read (src->fd, GST_BUFFER_DATA (buf) + bytes_read, to_read);
+    if (G_UNLIKELY (ret < 0)) {
+      if (errno == EAGAIN || errno == EINTR)
+        continue;
+      goto could_not_read;
+    }
+
+    /* files should eos if they read 0 and more was requested */
+    if (G_UNLIKELY (ret == 0)) {
+      /* .. but first we should return any remaining data */
+      if (bytes_read > 0)
+        break;
+      goto eos;
+    }
+
+    to_read -= ret;
+    bytes_read += ret;
+
+    src->read_position += ret;
+  }
+
+  GST_BUFFER_SIZE (buf) = bytes_read;
+  GST_BUFFER_OFFSET_END (buf) = offset + bytes_read;
 
   *buffer = buf;
 
-  src->read_position += length;
-
   return GST_FLOW_OK;
 
   /* ERROR */
@@ -872,16 +868,9 @@ could_not_read:
     gst_buffer_unref (buf);
     return GST_FLOW_ERROR;
   }
-unexpected_eos:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
-        ("unexpected end of file."));
-    gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
-  }
 eos:
   {
-    GST_DEBUG ("non-regular file hits EOS");
+    GST_DEBUG ("EOS");
     gst_buffer_unref (buf);
     return GST_FLOW_UNEXPECTED;
   }
@@ -894,7 +883,7 @@ gst_file_src_create (GstBaseSrc * basesrc, guint64 offset, guint length,
   GstFileSrc *src;
   GstFlowReturn ret;
 
-  src = GST_FILE_SRC (basesrc);
+  src = GST_FILE_SRC_CAST (basesrc);
 
 #ifdef HAVE_MMAP
   if (src->using_mmap) {
@@ -1034,6 +1023,8 @@ gst_file_src_start (GstBaseSrc * basesrc)
    * don't know their length, so seeking isn't useful/meaningful */
   src->seekable = src->seekable && src->is_regular;
 
+  gst_base_src_set_dynamic_size (basesrc, src->seekable);
+
   return TRUE;
 
   /* ERROR */
@@ -1113,7 +1104,7 @@ gst_file_src_uri_get_type (void)
 static gchar **
 gst_file_src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "file", NULL };
+  static gchar *protocols[] = { (char *) "file", NULL };
 
   return protocols;
 }
@@ -1132,6 +1123,7 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
   gchar *location, *hostname = NULL;
   gboolean ret = FALSE;
   GstFileSrc *src = GST_FILE_SRC (handler);
+  GError *error = NULL;
 
   if (strcmp (uri, "file://") == 0) {
     /* Special case for "file://" as this is used by some applications
@@ -1141,10 +1133,16 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return TRUE;
   }
 
-  location = g_filename_from_uri (uri, &hostname, NULL);
+  location = g_filename_from_uri (uri, &hostname, &error);
 
-  if (!location) {
-    GST_WARNING_OBJECT (src, "Invalid URI '%s' for filesrc", uri);
+  if (!location || error) {
+    if (error) {
+      GST_WARNING_OBJECT (src, "Invalid URI '%s' for filesrc: %s", uri,
+          error->message);
+      g_error_free (error);
+    } else {
+      GST_WARNING_OBJECT (src, "Invalid URI '%s' for filesrc", uri);
+    }
     goto beach;
   }
 
