@@ -179,7 +179,7 @@ gst_output_selector_init (GstOutputSelector * sel,
   /* srcpad management */
   sel->active_srcpad = NULL;
   sel->nb_srcpads = 0;
-  gst_segment_init (&sel->segment, GST_FORMAT_TIME);
+  gst_segment_init (&sel->segment, GST_FORMAT_UNDEFINED);
   sel->pending_srcpad = NULL;
 
   sel->resend_latest = FALSE;
@@ -294,10 +294,13 @@ gst_output_selector_get_property (GObject * object, guint prop_id,
 static GstCaps *
 gst_output_selector_sink_getcaps (GstPad * pad)
 {
-  GstOutputSelector *sel = GST_OUTPUT_SELECTOR (GST_PAD_PARENT (pad));
+  GstOutputSelector *sel = GST_OUTPUT_SELECTOR (gst_pad_get_parent (pad));
   GstPad *active = NULL;
   GstCaps *caps = NULL;
 
+  if (G_UNLIKELY (sel == NULL))
+    return gst_caps_new_any ();
+
   GST_OBJECT_LOCK (sel);
   if (sel->pending_srcpad)
     active = gst_object_ref (sel->pending_srcpad);
@@ -312,6 +315,8 @@ gst_output_selector_sink_getcaps (GstPad * pad)
   if (caps == NULL) {
     caps = gst_caps_new_any ();
   }
+
+  gst_object_unref (sel);
   return caps;
 }
 
@@ -363,8 +368,9 @@ gst_output_selector_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GstFlowReturn res;
   GstPad *allocpad;
 
-  sel = GST_OUTPUT_SELECTOR (GST_PAD_PARENT (pad));
-  res = GST_FLOW_NOT_LINKED;
+  sel = GST_OUTPUT_SELECTOR (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (sel == NULL))
+    return GST_FLOW_WRONG_STATE;
 
   GST_OBJECT_LOCK (sel);
   allocpad = sel->pending_srcpad ? sel->pending_srcpad : sel->active_srcpad;
@@ -390,6 +396,7 @@ gst_output_selector_buffer_alloc (GstPad * pad, guint64 offset, guint size,
 
   GST_DEBUG_OBJECT (sel, "buffer alloc finished: %s", gst_flow_get_name (res));
 
+  gst_object_unref (sel);
   return res;
 }
 
@@ -455,10 +462,12 @@ gst_output_selector_switch (GstOutputSelector * osel)
   osel->pending_srcpad = NULL;
   GST_OBJECT_UNLOCK (GST_OBJECT (osel));
 
-  /* Send NEWSEGMENT event and latest buffer if switching succeeded */
-  if (res) {
+  /* Send NEWSEGMENT event and latest buffer if switching succeeded
+   * and we already have a valid segment configured */
+  if (res && osel->segment.format != GST_FORMAT_UNDEFINED) {
     /* Send NEWSEGMENT to the pad we are going to switch to */
     seg = &osel->segment;
+
     /* If resending then mark newsegment start and position accordingly */
     if (osel->resend_latest && osel->latest_buffer &&
         GST_BUFFER_TIMESTAMP_IS_VALID (osel->latest_buffer)) {
@@ -579,6 +588,10 @@ gst_output_selector_handle_sink_event (GstPad * pad, GstEvent * event)
   GstPad *output_pad = NULL;
 
   sel = GST_OUTPUT_SELECTOR (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (sel == NULL)) {
+    gst_event_unref (event);
+    return FALSE;
+  }
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
