@@ -42,13 +42,12 @@ static void
 n_print (const char *format, ...)
 {
   va_list args;
-  gint retval;
 
   if (_name)
     g_print ("%s", _name);
 
   va_start (args, format);
-  retval = g_vprintf (format, args);
+  g_vprintf (format, args);
   va_end (args);
 }
 
@@ -172,34 +171,64 @@ print_event_masks (const GstEventMask * masks)
 }
 #endif
 
-static char *
-get_rank_name (gint rank)
+static const char *
+get_rank_name (char *s, gint rank)
 {
-  switch (rank) {
-    case GST_RANK_NONE:
-      return "none";
-    case GST_RANK_MARGINAL:
-      return "marginal";
-    case GST_RANK_SECONDARY:
-      return "secondary";
-    case GST_RANK_PRIMARY:
-      return "primary";
-    default:
-      return "unknown";
+  static const int ranks[4] = {
+    GST_RANK_NONE, GST_RANK_MARGINAL, GST_RANK_SECONDARY, GST_RANK_PRIMARY
+  };
+  static const char *rank_names[4] = { "none", "marginal", "secondary",
+    "primary"
+  };
+  int i;
+  int best_i;
+
+  best_i = 0;
+  for (i = 0; i < 4; i++) {
+    if (rank == ranks[i])
+      return rank_names[i];
+    if (abs (rank - ranks[i]) < abs (rank - ranks[best_i])) {
+      best_i = i;
+    }
   }
+
+  sprintf (s, "%s %c %d", rank_names[best_i],
+      (rank - ranks[best_i] > 0) ? '+' : '-', abs (ranks[best_i] - rank));
+
+  return s;
+}
+
+static gboolean
+print_factory_details_meta_data (GQuark field_id, const GValue * value,
+    gpointer user_data)
+{
+  gchar *val = g_strdup_value_contents (value);
+  gchar *key = g_strdup (g_quark_to_string (field_id));
+
+  key[0] = g_ascii_toupper (key[0]);
+  n_print ("  %s:\t\t%s\n", key, val);
+  g_free (val);
+  g_free (key);
+  return TRUE;
 }
 
 static void
 print_factory_details_info (GstElementFactory * factory)
 {
+  char s[20];
+
   n_print ("Factory Details:\n");
   n_print ("  Long name:\t%s\n", factory->details.longname);
   n_print ("  Class:\t%s\n", factory->details.klass);
   n_print ("  Description:\t%s\n", factory->details.description);
   n_print ("  Author(s):\t%s\n", factory->details.author);
   n_print ("  Rank:\t\t%s (%d)\n",
-      get_rank_name (GST_PLUGIN_FEATURE (factory)->rank),
+      get_rank_name (s, GST_PLUGIN_FEATURE (factory)->rank),
       GST_PLUGIN_FEATURE (factory)->rank);
+  if (factory->meta_data != NULL) {
+    gst_structure_foreach ((GstStructure *) factory->meta_data,
+        print_factory_details_meta_data, NULL);
+  }
   n_print ("\n");
 }
 
@@ -257,6 +286,47 @@ print_interfaces (GType type)
   }
 }
 
+static gchar *
+flags_to_string (GFlagsValue * vals, guint flags)
+{
+  GString *s = NULL;
+  guint flags_left, i;
+
+  /* first look for an exact match and count the number of values */
+  for (i = 0; vals[i].value_name != NULL; ++i) {
+    if (vals[i].value == flags)
+      return g_strdup (vals[i].value_nick);
+  }
+
+  s = g_string_new (NULL);
+
+  /* we assume the values are sorted from lowest to highest value */
+  flags_left = flags;
+  while (i > 0) {
+    --i;
+    if (vals[i].value != 0 && (flags_left & vals[i].value) == vals[i].value) {
+      if (s->len > 0)
+        g_string_append_c (s, '+');
+      g_string_append (s, vals[i].value_nick);
+      flags_left -= vals[i].value;
+      if (flags_left == 0)
+        break;
+    }
+  }
+
+  if (s->len == 0)
+    g_string_assign (s, "(none)");
+
+  return g_string_free (s, FALSE);
+}
+
+#define KNOWN_PARAM_FLAGS \
+  (G_PARAM_CONSTRUCT | G_PARAM_CONSTRUCT_ONLY | \
+  G_PARAM_LAX_VALIDATION |  G_PARAM_STATIC_STRINGS | \
+  G_PARAM_READABLE | G_PARAM_WRITABLE | GST_PARAM_CONTROLLABLE | \
+  GST_PARAM_MUTABLE_PLAYING | GST_PARAM_MUTABLE_PAUSED | \
+  GST_PARAM_MUTABLE_READY)
+
 static void
 print_element_properties_info (GstElement * element)
 {
@@ -286,57 +356,55 @@ print_element_properties_info (GstElement * element)
     if (param->flags & G_PARAM_READABLE) {
       g_object_get_property (G_OBJECT (element), param->name, &value);
       readable = TRUE;
-      if (!first_flag)
-        g_print (", ");
-      else
-        first_flag = FALSE;
-      g_print (_("readable"));
+      g_print ("%s%s", (first_flag) ? "" : ", ", _("readable"));
+      first_flag = FALSE;
+    } else {
+      /* if we can't read the property value, assume it's set to the default
+       * (which might not be entirely true for sub-classes, but that's an
+       * unlikely corner-case anyway) */
+      g_param_value_set_default (param, &value);
     }
     if (param->flags & G_PARAM_WRITABLE) {
-      if (!first_flag)
-        g_print (", ");
-      else
-        first_flag = FALSE;
-      g_print (_("writable"));
+      g_print ("%s%s", (first_flag) ? "" : ", ", _("writable"));
+      first_flag = FALSE;
     }
     if (param->flags & GST_PARAM_CONTROLLABLE) {
-      if (!first_flag)
-        g_print (", ");
-      g_print (_("controllable"));
+      g_print (", %s", _("controllable"));
+      first_flag = FALSE;
+    }
+    if (param->flags & GST_PARAM_MUTABLE_PLAYING) {
+      g_print (", %s", _("changeable in NULL, READY, PAUSED or PLAYING state"));
+    } else if (param->flags & GST_PARAM_MUTABLE_PAUSED) {
+      g_print (", %s", _("changeable only in NULL, READY or PAUSED state"));
+    } else if (param->flags & GST_PARAM_MUTABLE_READY) {
+      g_print (", %s", _("changeable only in NULL or READY state"));
+    }
+    if (param->flags & ~KNOWN_PARAM_FLAGS) {
+      g_print ("%s0x%0x", (first_flag) ? "" : ", ",
+          param->flags & ~KNOWN_PARAM_FLAGS);
     }
     n_print ("\n");
 
     switch (G_VALUE_TYPE (&value)) {
       case G_TYPE_STRING:
       {
-        GParamSpecString *pstring = G_PARAM_SPEC_STRING (param);
+        const char *string_val = g_value_get_string (&value);
 
         n_print ("%-23.23s String. ", "");
 
-        if (pstring->default_value == NULL)
-          g_print ("Default: null ");
+        if (string_val == NULL)
+          g_print ("Default: null");
         else
-          g_print ("Default: \"%s\" ", pstring->default_value);
-
-        if (readable) {
-          const char *string_val = g_value_get_string (&value);
-
-          if (string_val == NULL)
-            g_print ("Current: null");
-          else
-            g_print ("Current: \"%s\"", string_val);
-        }
+          g_print ("Default: \"%s\"", string_val);
         break;
       }
       case G_TYPE_BOOLEAN:
       {
-        GParamSpecBoolean *pboolean = G_PARAM_SPEC_BOOLEAN (param);
+        gboolean bool_val = g_value_get_boolean (&value);
 
         n_print ("%-23.23s Boolean. ", "");
-        g_print ("Default: %s ", (pboolean->default_value ? "true" : "false"));
-        if (readable)
-          g_print ("Current: %s",
-              (g_value_get_boolean (&value) ? "true" : "false"));
+
+        g_print ("Default: %s", bool_val ? "true" : "false");
         break;
       }
       case G_TYPE_ULONG:
@@ -345,9 +413,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Unsigned Long. ", "");
         g_print ("Range: %lu - %lu Default: %lu ",
-            pulong->minimum, pulong->maximum, pulong->default_value);
-        if (readable)
-          g_print ("Current: %lu", g_value_get_ulong (&value));
+            pulong->minimum, pulong->maximum, g_value_get_ulong (&value));
         break;
       }
       case G_TYPE_LONG:
@@ -356,9 +422,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Long. ", "");
         g_print ("Range: %ld - %ld Default: %ld ",
-            plong->minimum, plong->maximum, plong->default_value);
-        if (readable)
-          g_print ("Current: %ld", g_value_get_long (&value));
+            plong->minimum, plong->maximum, g_value_get_long (&value));
         break;
       }
       case G_TYPE_UINT:
@@ -367,9 +431,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Unsigned Integer. ", "");
         g_print ("Range: %u - %u Default: %u ",
-            puint->minimum, puint->maximum, puint->default_value);
-        if (readable)
-          g_print ("Current: %u", g_value_get_uint (&value));
+            puint->minimum, puint->maximum, g_value_get_uint (&value));
         break;
       }
       case G_TYPE_INT:
@@ -378,9 +440,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Integer. ", "");
         g_print ("Range: %d - %d Default: %d ",
-            pint->minimum, pint->maximum, pint->default_value);
-        if (readable)
-          g_print ("Current: %d", g_value_get_int (&value));
+            pint->minimum, pint->maximum, g_value_get_int (&value));
         break;
       }
       case G_TYPE_UINT64:
@@ -390,9 +450,7 @@ print_element_properties_info (GstElement * element)
         n_print ("%-23.23s Unsigned Integer64. ", "");
         g_print ("Range: %" G_GUINT64_FORMAT " - %" G_GUINT64_FORMAT
             " Default: %" G_GUINT64_FORMAT " ",
-            puint64->minimum, puint64->maximum, puint64->default_value);
-        if (readable)
-          g_print ("Current: %" G_GUINT64_FORMAT, g_value_get_uint64 (&value));
+            puint64->minimum, puint64->maximum, g_value_get_uint64 (&value));
         break;
       }
       case G_TYPE_INT64:
@@ -402,9 +460,7 @@ print_element_properties_info (GstElement * element)
         n_print ("%-23.23s Integer64. ", "");
         g_print ("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT
             " Default: %" G_GINT64_FORMAT " ",
-            pint64->minimum, pint64->maximum, pint64->default_value);
-        if (readable)
-          g_print ("Current: %" G_GINT64_FORMAT, g_value_get_int64 (&value));
+            pint64->minimum, pint64->maximum, g_value_get_int64 (&value));
         break;
       }
       case G_TYPE_FLOAT:
@@ -413,9 +469,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Float. ", "");
         g_print ("Range: %15.7g - %15.7g Default: %15.7g ",
-            pfloat->minimum, pfloat->maximum, pfloat->default_value);
-        if (readable)
-          g_print ("Current: %15.7g", g_value_get_float (&value));
+            pfloat->minimum, pfloat->maximum, g_value_get_float (&value));
         break;
       }
       case G_TYPE_DOUBLE:
@@ -424,9 +478,7 @@ print_element_properties_info (GstElement * element)
 
         n_print ("%-23.23s Double. ", "");
         g_print ("Range: %15.7g - %15.7g Default: %15.7g ",
-            pdouble->minimum, pdouble->maximum, pdouble->default_value);
-        if (readable)
-          g_print ("Current: %15.7g", g_value_get_double (&value));
+            pdouble->minimum, pdouble->maximum, g_value_get_double (&value));
         break;
       }
       default:
@@ -439,27 +491,22 @@ print_element_properties_info (GstElement * element)
             print_caps (caps, "                           ");
           }
         } else if (G_IS_PARAM_SPEC_ENUM (param)) {
-          GParamSpecEnum *penum = G_PARAM_SPEC_ENUM (param);
           GEnumValue *values;
           guint j = 0;
           gint enum_value;
-          const gchar *def_val_nick = "", *cur_val_nick = "";
+          const gchar *value_nick = "";
 
           values = G_ENUM_CLASS (g_type_class_ref (param->value_type))->values;
           enum_value = g_value_get_enum (&value);
 
           while (values[j].value_name) {
             if (values[j].value == enum_value)
-              cur_val_nick = values[j].value_nick;
-            if (values[j].value == penum->default_value)
-              def_val_nick = values[j].value_nick;
+              value_nick = values[j].value_nick;
             j++;
           }
 
-          n_print
-              ("%-23.23s Enum \"%s\" Default: %d, \"%s\" Current: %d, \"%s\"",
-              "", g_type_name (G_VALUE_TYPE (&value)), penum->default_value,
-              def_val_nick, enum_value, cur_val_nick);
+          n_print ("%-23.23s Enum \"%s\" Default: %d, \"%s\"", "",
+              g_type_name (G_VALUE_TYPE (&value)), enum_value, value_nick);
 
           j = 0;
           while (values[j].value_name) {
@@ -473,54 +520,27 @@ print_element_properties_info (GstElement * element)
           /* g_type_class_unref (ec); */
         } else if (G_IS_PARAM_SPEC_FLAGS (param)) {
           GParamSpecFlags *pflags = G_PARAM_SPEC_FLAGS (param);
-          GFlagsValue *values;
-          guint j = 0;
-          gint flags_value;
-          GString *cur_flags = NULL, *def_flags = NULL;
+          GFlagsValue *vals;
+          gchar *cur;
 
-          values = G_FLAGS_CLASS (g_type_class_ref (param->value_type))->values;
-          flags_value = g_value_get_flags (&value);
+          vals = pflags->flags_class->values;
 
-          while (values[j].value_name) {
-            if (values[j].value & flags_value) {
-              if (cur_flags) {
-                g_string_append_printf (cur_flags, " | %s",
-                    values[j].value_nick);
-              } else {
-                cur_flags = g_string_new (values[j].value_nick);
-              }
-            }
-            if (values[j].value & pflags->default_value) {
-              if (def_flags) {
-                g_string_append_printf (def_flags, " | %s",
-                    values[j].value_nick);
-              } else {
-                def_flags = g_string_new (values[j].value_nick);
-              }
-            }
-            j++;
-          }
+          cur = flags_to_string (vals, g_value_get_flags (&value));
 
-          n_print
-              ("%-23.23s Flags \"%s\" Default: 0x%08x, \"%s\" Current: 0x%08x, \"%s\"",
-              "", g_type_name (G_VALUE_TYPE (&value)), pflags->default_value,
-              (def_flags ? def_flags->str : "(none)"), flags_value,
-              (cur_flags ? cur_flags->str : "(none)"));
+          n_print ("%-23.23s Flags \"%s\" Default: 0x%08x, \"%s\"", "",
+              g_type_name (G_VALUE_TYPE (&value)),
+              g_value_get_flags (&value), cur);
 
-          j = 0;
-          while (values[j].value_name) {
+          while (vals[0].value_name) {
             g_print ("\n");
             if (_name)
               g_print ("%s", _name);
             g_print ("%-23.23s    (0x%08x): %-16s - %s", "",
-                values[j].value, values[j].value_nick, values[j].value_name);
-            j++;
+                vals[0].value, vals[0].value_nick, vals[0].value_name);
+            ++vals;
           }
 
-          if (cur_flags)
-            g_string_free (cur_flags, TRUE);
-          if (def_flags)
-            g_string_free (def_flags, TRUE);
+          g_free (cur);
         } else if (G_IS_PARAM_SPEC_OBJECT (param)) {
           n_print ("%-23.23s Object of type \"%s\"", "",
               g_type_name (param->value_type));
@@ -551,12 +571,8 @@ print_element_properties_info (GstElement * element)
           g_print ("Range: %d/%d - %d/%d Default: %d/%d ",
               pfraction->min_num, pfraction->min_den,
               pfraction->max_num, pfraction->max_den,
-              pfraction->def_num, pfraction->def_den);
-          if (readable)
-            g_print ("Current: %d/%d",
-                gst_value_get_fraction_numerator (&value),
-                gst_value_get_fraction_denominator (&value));
-
+              gst_value_get_fraction_numerator (&value),
+              gst_value_get_fraction_denominator (&value));
         } else if (GST_IS_PARAM_SPEC_MINI_OBJECT (param)) {
           n_print ("%-23.23s MiniObject of type \"%s\"", "",
               g_type_name (param->value_type));
@@ -648,22 +664,28 @@ print_element_flag_info (GstElement * element)
 static void
 print_implementation_info (GstElement * element)
 {
-  GstObjectClass *gstobject_class;
-  GstElementClass *gstelement_class;
-
-  gstobject_class = GST_OBJECT_CLASS (G_OBJECT_GET_CLASS (element));
-  gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
-
   n_print ("\n");
   n_print ("Element Implementation:\n");
 
-  n_print ("  Has change_state() function: %s\n",
-      GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
+  {
+    GstElementClass *gstelement_class;
+
+    gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
+
+    n_print ("  Has change_state() function: %s\n",
+        GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
+  }
 #ifndef GST_DISABLE_LOADSAVE
-  n_print ("  Has custom save_thyself() function: %s\n",
-      GST_DEBUG_FUNCPTR_NAME (gstobject_class->save_thyself));
-  n_print ("  Has custom restore_thyself() function: %s\n",
-      GST_DEBUG_FUNCPTR_NAME (gstobject_class->restore_thyself));
+  {
+    GstObjectClass *gstobject_class;
+
+    gstobject_class = GST_OBJECT_CLASS (G_OBJECT_GET_CLASS (element));
+
+    n_print ("  Has custom save_thyself() function: %s\n",
+        GST_DEBUG_FUNCPTR_NAME (gstobject_class->save_thyself));
+    n_print ("  Has custom restore_thyself() function: %s\n",
+        GST_DEBUG_FUNCPTR_NAME (gstobject_class->restore_thyself));
+  }
 #endif
 }
 
@@ -790,18 +812,36 @@ print_pad_info (GstElement * element)
       n_print ("      Has custom queryfunc(): %s\n",
           GST_DEBUG_FUNCPTR_NAME (pad->queryfunc));
     if (pad->querytypefunc != gst_pad_get_query_types_default) {
-      n_print ("        Provides query types:\n");
-      print_query_types (gst_pad_get_query_types (pad));
+      const GstQueryType *query_types = gst_pad_get_query_types (pad);
+      if (query_types) {
+        n_print ("        Provides query types:\n");
+        print_query_types (query_types);
+      }
     }
 
-    if (pad->intlinkfunc != gst_pad_get_internal_links_default)
-      n_print ("      Has custom intconnfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (pad->intlinkfunc));
+    if (pad->iterintlinkfunc != gst_pad_iterate_internal_links_default)
+      n_print ("      Has custom iterintlinkfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->iterintlinkfunc));
 
     if (pad->bufferallocfunc)
       n_print ("      Has bufferallocfunc(): %s\n",
           GST_DEBUG_FUNCPTR_NAME (pad->bufferallocfunc));
 
+    if (pad->getcapsfunc)
+      n_print ("      Has getcapsfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->getcapsfunc));
+    if (pad->setcapsfunc)
+      n_print ("      Has setcapsfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->setcapsfunc));
+    /* gst_pad_acceptcaps_default is static :/ */
+    if (pad->acceptcapsfunc)
+      n_print ("      Has acceptcapsfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->acceptcapsfunc));
+    if (pad->fixatecapsfunc)
+      n_print ("      Has fixatecapsfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->fixatecapsfunc));
+
+
     if (pad->padtemplate)
       n_print ("    Pad Template: '%s'\n", pad->padtemplate->name_template);
 
@@ -812,13 +852,19 @@ print_pad_info (GstElement * element)
   }
 }
 
-#if 0
-static gint
-compare_signal_names (GSignalQuery * a, GSignalQuery * b)
+static gboolean
+has_sometimes_template (GstElement * element)
 {
-  return strcmp (a->signal_name, b->signal_name);
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
+  GList *l;
+
+  for (l = klass->padtemplates; l != NULL; l = l->next) {
+    if (GST_PAD_TEMPLATE (l->data)->presence == GST_PAD_SOMETIMES)
+      return TRUE;
+  }
+
+  return FALSE;
 }
-#endif
 
 static void
 print_signal_info (GstElement * element)
@@ -833,6 +879,22 @@ print_signal_info (GstElement * element)
 
   for (k = 0; k < 2; k++) {
     found_signals = NULL;
+
+    /* For elements that have sometimes pads, also list a few useful GstElement
+     * signals. Put these first, so element-specific ones come later. */
+    if (k == 0 && has_sometimes_template (element)) {
+      query = g_new0 (GSignalQuery, 1);
+      g_signal_query (g_signal_lookup ("pad-added", GST_TYPE_ELEMENT), query);
+      found_signals = g_slist_append (found_signals, query);
+      query = g_new0 (GSignalQuery, 1);
+      g_signal_query (g_signal_lookup ("pad-removed", GST_TYPE_ELEMENT), query);
+      found_signals = g_slist_append (found_signals, query);
+      query = g_new0 (GSignalQuery, 1);
+      g_signal_query (g_signal_lookup ("no-more-pads", GST_TYPE_ELEMENT),
+          query);
+      found_signals = g_slist_append (found_signals, query);
+    }
+
     for (type = G_OBJECT_TYPE (element); type; type = g_type_parent (type)) {
       if (type == GST_TYPE_ELEMENT || type == GST_TYPE_OBJECT)
         break;
@@ -932,10 +994,35 @@ print_children_info (GstElement * element)
   }
 }
 
+static void
+print_blacklist (void)
+{
+  GList *plugins, *cur;
+  gint count = 0;
+
+  g_print ("%s\n", _("Blacklisted files:"));
+
+  plugins = gst_default_registry_get_plugin_list ();
+  for (cur = plugins; cur != NULL; cur = g_list_next (cur)) {
+    GstPlugin *plugin = (GstPlugin *) (cur->data);
+    if (plugin->flags & GST_PLUGIN_FLAG_BLACKLISTED) {
+      g_print ("  %s\n", plugin->desc.name);
+      count++;
+    }
+  }
+
+  g_print ("\n");
+  g_print (_("Total count: "));
+  g_print (ngettext ("%d blacklisted file", "%d blacklisted files", count),
+      count);
+  g_print ("\n");
+  gst_plugin_list_free (plugins);
+}
+
 static void
 print_element_list (gboolean print_all)
 {
-  int plugincount = 0, featurecount = 0;
+  int plugincount = 0, featurecount = 0, blacklistcount = 0;
   GList *plugins, *orig_plugins;
 
   orig_plugins = plugins = gst_default_registry_get_plugin_list ();
@@ -947,12 +1034,19 @@ print_element_list (gboolean print_all)
     plugins = g_list_next (plugins);
     plugincount++;
 
+    if (plugin->flags & GST_PLUGIN_FLAG_BLACKLISTED) {
+      blacklistcount++;
+      continue;
+    }
+
     orig_features = features =
         gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
         plugin->desc.name);
     while (features) {
       GstPluginFeature *feature;
 
+      if (G_UNLIKELY (features->data == NULL))
+        goto next;
       feature = GST_PLUGIN_FEATURE (features->data);
       featurecount++;
 
@@ -1001,6 +1095,7 @@ print_element_list (gboolean print_all)
               g_type_name (G_OBJECT_TYPE (feature)));
       }
 
+    next:
       features = g_list_next (features);
     }
 
@@ -1012,6 +1107,12 @@ print_element_list (gboolean print_all)
   g_print ("\n");
   g_print (_("Total count: "));
   g_print (ngettext ("%d plugin", "%d plugins", plugincount), plugincount);
+  if (blacklistcount) {
+    g_print (" (");
+    g_print (ngettext ("%d blacklist entry", "%d blacklist entries",
+            blacklistcount), blacklistcount);
+    g_print (" not shown)");
+  }
   g_print (", ");
   g_print (ngettext ("%d feature", "%d features", featurecount), featurecount);
   g_print ("\n");
@@ -1072,12 +1173,12 @@ print_all_uri_handlers (void)
               gst_uri_handler_get_protocols (GST_URI_HANDLER (element));
           joined = g_strjoinv (", ", uri_protocols);
 
-          g_print ("%s (%s, rank %u): %s\n", GST_OBJECT_NAME (factory), dir,
+          g_print ("%s (%s, rank %u): %s\n",
+              gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (factory)), dir,
               gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE (factory)),
               joined);
 
           g_free (joined);
-          //g_strfreev (uri_protocols);
         }
 
         gst_object_unref (element);
@@ -1102,6 +1203,25 @@ print_plugin_info (GstPlugin * plugin)
   n_print ("  Version:\t\t%s\n", plugin->desc.version);
   n_print ("  License:\t\t%s\n", plugin->desc.license);
   n_print ("  Source module:\t%s\n", plugin->desc.source);
+  if (plugin->desc.release_datetime != NULL) {
+    const gchar *tz = "(UTC)";
+    gchar *str, *sep;
+
+    /* may be: YYYY-MM-DD or YYYY-MM-DDTHH:MMZ */
+    /* YYYY-MM-DDTHH:MMZ => YYYY-MM-DD HH:MM (UTC) */
+    str = g_strdup (plugin->desc.release_datetime);
+    sep = strstr (str, "T");
+    if (sep != NULL) {
+      *sep = ' ';
+      sep = strstr (sep + 1, "Z");
+      if (sep != NULL)
+        *sep = ' ';
+    } else {
+      tz = "";
+    }
+    n_print ("  Source release date:\t%s%s\n", str, tz);
+    g_free (str);
+  }
   n_print ("  Binary package:\t%s\n", plugin->desc.package);
   n_print ("  Origin URL:\t\t%s\n", plugin->desc.origin);
   n_print ("\n");
@@ -1113,7 +1233,7 @@ print_plugin_features (GstPlugin * plugin)
   GList *features;
   gint num_features = 0;
   gint num_elements = 0;
-  gint num_types = 0;
+  gint num_typefinders = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
@@ -1157,8 +1277,8 @@ print_plugin_features (GstPlugin * plugin)
         g_print ("%s: %s: no extensions\n", plugin->desc.name,
             gst_plugin_feature_get_name (feature));
 
-      num_types++;
-    } else {
+      num_typefinders++;
+    } else if (feature) {
       n_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
           g_type_name (G_OBJECT_TYPE (feature)));
       num_other++;
@@ -1170,8 +1290,8 @@ print_plugin_features (GstPlugin * plugin)
   n_print ("  %d features:\n", num_features);
   if (num_elements > 0)
     n_print ("  +-- %d elements\n", num_elements);
-  if (num_types > 0)
-    n_print ("  +-- %d types\n", num_types);
+  if (num_typefinders > 0)
+    n_print ("  +-- %d typefinders\n", num_typefinders);
   if (num_indexes > 0)
     n_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
@@ -1386,19 +1506,40 @@ print_plugin_automatic_install_info (GstPlugin * plugin)
   g_list_free (features);
 }
 
+static void
+print_all_plugin_automatic_install_info (void)
+{
+  GList *plugins, *orig_plugins;
+
+  orig_plugins = plugins = gst_default_registry_get_plugin_list ();
+  while (plugins) {
+    GstPlugin *plugin;
+
+    plugin = (GstPlugin *) (plugins->data);
+    plugins = g_list_next (plugins);
+
+    print_plugin_automatic_install_info (plugin);
+  }
+  gst_plugin_list_free (orig_plugins);
+}
+
 int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
+  gboolean do_print_blacklist = FALSE;
   gboolean plugin_name = FALSE;
   gboolean print_aii = FALSE;
   gboolean uri_handlers = FALSE;
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionEntry options[] = {
     {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
+    {"print-blacklist", 'b', 0, G_OPTION_ARG_NONE, &do_print_blacklist,
+        N_("Print list of blacklisted files"), NULL},
     {"print-plugin-auto-install-info", '\0', 0, G_OPTION_ARG_NONE, &print_aii,
         N_("Print a machine-parsable list of features the specified plugin "
-              "provides.\n                                       "
+              "or all plugins provide.\n                                       "
               "Useful in connection with external automatic plugin "
               "installation mechanisms"), NULL},
     {"plugin", '\0', 0, G_OPTION_ARG_NONE, &plugin_name,
@@ -1412,6 +1553,7 @@ main (int argc, char *argv[])
   };
   GOptionContext *ctx;
   GError *err = NULL;
+#endif
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -1419,9 +1561,13 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  if (!g_thread_supported ())
-    g_thread_init (NULL);
+#if !GLIB_CHECK_VERSION (2, 31, 0)
+  g_thread_init (NULL);
+#endif
 
+  gst_tools_set_prgname ("gst-inspect");
+
+#ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
@@ -1430,6 +1576,9 @@ main (int argc, char *argv[])
     exit (1);
   }
   g_option_context_free (ctx);
+#else
+  gst_init (&argc, &argv);
+#endif
 
   gst_tools_print_version ("gst-inspect");
 
@@ -1447,7 +1596,14 @@ main (int argc, char *argv[])
   if (uri_handlers) {
     print_all_uri_handlers ();
   } else if (argc == 1 || print_all) {
-    print_element_list (print_all);
+    if (do_print_blacklist)
+      print_blacklist ();
+    else {
+      if (print_aii)
+        print_all_plugin_automatic_install_info ();
+      else
+        print_element_list (print_all);
+    }
   } else {
     /* else we try to get a factory */
     GstElementFactory *factory;
